@@ -4,10 +4,10 @@ a generic Qubole command and the implementation of all
 the specific commands
 """
 
-from qubole import Qubole
-from resource import Resource
-from exception import ParseError
-from account import Account
+from qds_sdk.qubole import Qubole
+from qds_sdk.resource import Resource
+from qds_sdk.exception import ParseError
+from qds_sdk.account import Account
 from qds_sdk.util import GentleOptionParser
 from qds_sdk.util import OptionParsingError
 from qds_sdk.util import OptionParsingExit
@@ -147,7 +147,17 @@ class Command(Resource):
 
         r = conn.get(result_path, {'inline': inline})
         if r.get('inline'):
-            fp.write(r['results'].encode('utf8'))
+            if sys.version_info < (3, 0, 0):
+                fp.write(r['results'].encode('utf8'))
+            else:
+                import io
+                if isinstance(fp, io.TextIOBase):
+                    fp.buffer.write(r['results'].encode('utf8'))
+                elif isinstance(fp, io.BufferedIOBase) or isinstance(fp, io.RawIOBase):
+                    fp.write(r['results'].encode('utf8'))
+                else:
+                    # Can this happen? Don't know what's the right thing to do in this case.
+                    pass
         else:
             acc = Account.find()
             boto_conn = boto.connect_s3(aws_access_key_id=acc.storage_access_key,
@@ -157,6 +167,7 @@ class Command(Resource):
             #fetch latest value of num_result_dir
             num_result_dir = Command.find(self.id).num_result_dir
             for s3_path in r['result_location']:
+                # In Python 3, in this case, `fp` should always be binary mode.
                 _download_to_local(boto_conn, s3_path, fp, num_result_dir, delim=delim)
 
 
@@ -182,6 +193,9 @@ class HiveCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this query")
+
     @classmethod
     def parse(cls, args):
         """
@@ -222,7 +236,7 @@ class HiveCommand(Command):
 
                 try:
                     q = open(options.script_location).read()
-                except IOError, e:
+                except IOError as e:
                     raise ParseError("Unable to open script location: %s" %
                                      str(e),
                                      cls.optparser.format_help())
@@ -254,7 +268,10 @@ class PrestoCommand(Command):
 
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
- 
+
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this query")
+
     @classmethod
     def parse(cls, args):
         """
@@ -294,7 +311,7 @@ class PrestoCommand(Command):
                 # script location is local file
                 try:
                     q = open(options.script_location).read()
-                except IOError, e:
+                except IOError as e:
                     raise ParseError("Unable to open script location: %s" %
                                      str(e),
                                      cls.optparser.format_help())
@@ -319,6 +336,9 @@ class HadoopCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     optparser.disable_interspersed_args()
 
     @classmethod
@@ -347,6 +367,7 @@ class HadoopCommand(Command):
 
         parsed['label'] = options.label
         parsed['can_notify'] = options.can_notify
+        parsed['name'] = options.name
         parsed["command_type"] = "HadoopCommand"
 
         if len(args) < 2:
@@ -375,7 +396,7 @@ class ShellCommand(Command):
     optparser.add_option("-i", "--files", dest="files",
                          help="List of files [optional] Format : file1,file2 (files in s3 bucket) These files will be copied to the working directory where the command is executed")
 
-    optparser.add_option("-a", "--archive", dest="archive",
+    optparser.add_option("-a", "--archives", dest="archives",
                          help="List of archives [optional] Format : archive1,archive2 (archives in s3 bucket) These are unarchived in the working directory where the command is executed")
 
     optparser.add_option("--cluster-label", dest="label",
@@ -384,6 +405,9 @@ class ShellCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     @classmethod
     def parse(cls, args):
         """
@@ -424,7 +448,7 @@ class ShellCommand(Command):
 
                 try:
                     s = open(options.script_location).read()
-                except IOError, e:
+                except IOError as e:
                     raise ParseError("Unable to open script location: %s" %
                                      str(e),
                                      cls.optparser.format_help())
@@ -468,6 +492,9 @@ class PigCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     @classmethod
     def parse(cls, args):
         """
@@ -508,7 +535,7 @@ class PigCommand(Command):
 
                 try:
                     s = open(options.script_location).read()
-                except IOError, e:
+                except IOError as e:
                     raise ParseError("Unable to open script location: %s" %
                                      str(e),
                                      cls.optparser.format_help())
@@ -574,6 +601,9 @@ class DbExportCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     @classmethod
     def parse(cls, args):
         """
@@ -665,6 +695,9 @@ class DbImportCommand(Command):
     optparser.add_option("--notify", action="store_true", dest="can_notify",
                          default=False, help="sends an email on command completion")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     @classmethod
     def parse(cls, args):
         """
@@ -705,7 +738,7 @@ class DbImportCommand(Command):
 
 class CompositeCommand(Command):
     @classmethod
-    def compose(cls, sub_commands, macros=None, cluster_label=None, notify=False):
+    def compose(cls, sub_commands, macros=None, cluster_label=None, notify=False, name=None):
         """
         Args:
             `sub_commands`: list of sub-command dicts
@@ -726,7 +759,8 @@ class CompositeCommand(Command):
                 "command_type": "CompositeCommand",
                 "macros": macros,
                 "label": cluster_label,
-                "can_notify": notify
+                "can_notify": notify,
+                "name": name
                }
 
 
@@ -742,6 +776,9 @@ class DbTapQueryCommand(Command):
     optparser.add_option("--macros", dest="macros",
                          help="expressions to expand macros used in query")
 
+    optparser.add_option("--name", dest="name",
+                         help="Assign a name to this command")
+
     @classmethod
     def parse(cls, args):
         """
@@ -783,7 +820,7 @@ def _read_iteratively(key_instance, fp, delim):
     while True:
         try:
             # Default buffer size is 8192 bytes
-            data = key_instance.next()
+            data = next(key_instance)
             fp.write(str(data).replace(chr(1), delim))
         except StopIteration:
             # Stream closes itself when the exception is raised
@@ -830,7 +867,7 @@ def _download_to_local(boto_conn, s3_path, fp, num_result_dir, delim=None):
             unique_paths.add(dir)
             if len(path) > 1:
                 file = int(path[1])
-                if files.has_key(dir) is False:
+                if dir not in files:
                     files[dir] = []
                 files[dir].append(file)
         if len(unique_paths) < num_result_dir:
